@@ -36,7 +36,7 @@ except ImportError:
     gzip = None
     GZIP_BASE = object
 
-__version__ = '1.0.1'
+__version__ = '1.0.2'
 
 
 class FakeShutdownEvent(object):
@@ -225,6 +225,10 @@ class SpeedtestException(Exception):
     """Base exception for this module"""
 
 
+class SpeedtestCLIError(SpeedtestException):
+    """Generic exception for raising errors during CLI operation"""
+
+
 class SpeedtestHTTPError(SpeedtestException):
     """Base HTTP exception for this module"""
 
@@ -293,7 +297,13 @@ class GzipDecodedResponse(GZIP_BASE):
             raise SpeedtestHTTPError('HTTP response body is gzip encoded, '
                                      'but gzip support is not available')
         IO = BytesIO or StringIO
-        self.io = IO(response.read())
+        self.io = IO()
+        while 1:
+            chunk = response.read(1024)
+            if len(chunk) == 0:
+                break
+            self.io.write(chunk)
+        self.io.seek(0)
         gzip.GzipFile.__init__(self, mode='rb', fileobj=self.io)
 
     def close(self):
@@ -303,6 +313,13 @@ class GzipDecodedResponse(GZIP_BASE):
             self.io.close()
 
 
+def get_exception():
+    """Helper function to work with py2.4-py3 for getting the current
+    exception in a try/except block
+    """
+    return sys.exc_info()[1]
+
+
 def bound_socket(*args, **kwargs):
     """Bind socket to a specified source IP address"""
 
@@ -398,7 +415,7 @@ def catch_request(request):
         uh = urlopen(request)
         return uh, False
     except HTTP_ERRORS:
-        e = sys.exc_info()[1]
+        e = get_exception()
         return None, e
 
 
@@ -710,7 +727,7 @@ class Speedtest(object):
         stream = get_response_stream(uh)
 
         while 1:
-            configxml.append(stream.read(10240))
+            configxml.append(stream.read(1024))
             if len(configxml[-1]) == 0:
                 break
         stream.close()
@@ -822,7 +839,7 @@ class Speedtest(object):
 
                 serversxml = []
                 while 1:
-                    serversxml.append(stream.read(10240))
+                    serversxml.append(stream.read(1024))
                     if len(serversxml[-1]) == 0:
                         break
 
@@ -982,7 +999,7 @@ class Speedtest(object):
                     r = h.getresponse()
                     total = (timeit.default_timer() - start)
                 except HTTP_ERRORS:
-                    e = sys.exc_info()[1]
+                    e = get_exception()
                     printer('%r' % e, debug=True)
                     cum.append(3600)
                     continue
@@ -1310,16 +1327,16 @@ def shell():
     printer('Retrieving speedtest.net configuration...', quiet)
     try:
         speedtest = Speedtest()
-    except ConfigRetrievalError:
+    except (ConfigRetrievalError, HTTP_ERRORS):
         printer('Cannot retrieve speedtest configuration')
-        sys.exit(1)
+        raise SpeedtestCLIError(get_exception())
 
     if args.list:
         try:
             speedtest.get_servers()
-        except ServersRetrievalError:
+        except (ServersRetrievalError, HTTP_ERRORS):
             print_('Cannot retrieve speedtest server list')
-            sys.exit(1)
+            raise SpeedtestCLIError(get_exception())
 
         for _, servers in sorted(speedtest.servers.items()):
             for server in servers:
@@ -1328,7 +1345,7 @@ def shell():
                 try:
                     print_(line)
                 except IOError:
-                    e = sys.exc_info()[1]
+                    e = get_exception()
                     if e.errno != errno.EPIPE:
                         raise
         sys.exit(0)
@@ -1346,14 +1363,13 @@ def shell():
         try:
             speedtest.get_servers(servers)
         except NoMatchedServers:
-            print_('No matched servers: %s' % args.server)
-            sys.exit(1)
-        except ServersRetrievalError:
+            raise SpeedtestCLIError('No matched servers: %s' % args.server)
+        except (ServersRetrievalError, HTTP_ERRORS):
             print_('Cannot retrieve speedtest server list')
-            sys.exit(1)
+            raise SpeedtestCLIError(get_exception())
         except InvalidServerIDType:
-            print_('%s is an invalid server type, must be int' % args.server)
-            sys.exit(1)
+            raise SpeedtestCLIError('%s is an invalid server type, must '
+                                    'be an int' % args.server)
 
         printer('Selecting best server based on ping...', quiet)
         speedtest.get_best_server()
@@ -1403,7 +1419,7 @@ def main():
     except KeyboardInterrupt:
         print_('\nCancelling...')
     except (SpeedtestException, SystemExit):
-        e = sys.exc_info()[1]
+        e = get_exception()
         if getattr(e, 'code', 1) != 0:
             raise SystemExit('ERROR: %s' % e)
 
